/***************************************************************************
                          artscontrolapplet_private.h  -  description
                             -------------------
    begin                : Don Jan 30 2003
    copyright            : (C) 2003 by Arnold Krille
    email                : arnold@arnoldarts.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ARTSCONTROLAPPLET_PRIVATE_H
#define ARTSCONTROLAPPLET_PRIVATE_H

#include "artscontrolapplet.h"

#include <qobject.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <kpopupmenu.h>
#include <kaction.h>
#include <qfont.h>

#include <kartsserver.h>
#include <kartsdispatcher.h>
#include <kartsfloatwatch.h>
#include <kartswidget.h>
#include <artsflow.h>
#include <connect.h>
#include <artsmoduleseffects.h>

#include "audiomanager.h"
#include "fftscopeview.h"
#include "statusview.h"
#include "midimanagerview.h"
#include "environmentview.h"
#include "mediatypesview.h"

class VolumeSlider;

class ArtsControlAppletPrivate : public QObject {
   Q_OBJECT
private:
	ArtsControlApplet *_parent;
public:
	QBoxLayout *layout;
	KArtsServer *arts;
	KArtsDispatcher *dispatcher;
	Arts::StereoVolumeControl volume;
	bool barts, bInUpdate;
	Gui_AUDIO_MANAGER *am;
	FFTScopeView *sv, *svinline;
	ArtsStatusView *asv;
	MidiManagerView *mmv;
	EnvironmentView *ev;
	MediaTypesView *mtv;
	VolumeSlider *volumeslider;

	KArtsWidget *vuw;
	Arts::StereoVolumeControlGui vu;

	KPopupMenu *menu;
	KAction *_showSV, *_showSVinline, *_showAM, *_showArtsStatus, *_showMidiManager, *_showEnvironment, *_showMediaTypes, *_moreBars, *_lessBars;
	KAction *_styleNormalBars, *_styleFireBars, *_styleLineBars, *_styleLEDs, *_styleAnalog, *_styleSmall;

	ArtsControlAppletPrivate( ArtsControlApplet *parent )
	    : QObject(parent)
	    , _parent( parent )
	    , barts( false )
	    , bInUpdate( false )
	{
		arts = new KArtsServer( 0 );
		dispatcher = new KArtsDispatcher( 0 );
		if( ! arts->server().isNull() ) barts = true;
		if( barts ) volume = arts->server().outVolume();
		am = 0;
		sv = 0;
		svinline=0;
		asv = 0; mmv = 0; ev = 0; mtv = 0;

		menu = new KPopupMenu( 0 );
		_showSV = new KAction( i18n( "Show FFTScope" ), "artscontrol", KShortcut(), this, SLOT( SV() ), this );
		_showSV->plug( menu );
		_showSVinline = new KAction( i18n( "Show FFTScope inline" ), "artscontrol", KShortcut(), this, SLOT( SVinline() ), this );
		_showSVinline->plug( menu );
		_showAM = new KAction( i18n( "Show Audiomanager" ), "artscontrol", KShortcut(), this, SLOT( AM() ), this );
		_showAM->plug( menu );
		_showArtsStatus = new KAction( i18n( "Show Arts Status" ), "artscontrol", KShortcut(), this, SLOT( ASV() ), this );
		_showArtsStatus->plug( menu );
		_showMidiManager = new KAction( i18n( "Show Midimanager" ), "artscontrol", KShortcut(), this, SLOT( MMV() ), this );
		_showMidiManager->plug( menu );
		_showEnvironment = new KAction( i18n( "Show Environment" ), "artscontrol", KShortcut(), this, SLOT( EV() ), this );
		_showEnvironment->plug( menu );
		_showMediaTypes = new KAction( i18n( "Show Available Mediatypes" ), "artscontrol", KShortcut(), this, SLOT( MTV() ), this );
		_showMediaTypes->plug( menu );
		menu->insertSeparator();
		_moreBars = new KAction( i18n( "More Bars in the VU-Meters" ), "up", KShortcut(), this, SLOT( moreBars() ), this );
		_moreBars->plug( menu );
		_lessBars = new KAction( i18n( "Less Bars in the VU-Meters" ), "down", KShortcut(), this, SLOT( lessBars() ), this );
		_lessBars->plug( menu );
		menu->insertSeparator();
		_styleNormalBars = new KAction( i18n( "Style: NormalBars" ), "", KShortcut(), this, SLOT( styleNormalBars() ), this ); _styleNormalBars->plug( menu );
		_styleFireBars = new KAction( i18n( "Style: FireBars" ), "", KShortcut(), this, SLOT( styleFireBars() ), this ); _styleFireBars->plug( menu );
		_styleLineBars = new KAction( i18n( "Style: LineBars" ), "", KShortcut(), this, SLOT( styleLineBars() ), this ); _styleLineBars->plug( menu );
		_styleLEDs = new KAction( i18n( "Style: LEDs" ), "", KShortcut(), this, SLOT( styleLEDs() ), this ); _styleLEDs->plug( menu );
		_styleAnalog = new KAction( i18n( "Style: Analog" ), "", KShortcut(), this, SLOT( styleAnalog() ), this ); _styleAnalog->plug( menu );
		_styleSmall = new KAction( i18n( "Style: Small" ), "", KShortcut(), this, SLOT( styleSmall() ), this ); _styleSmall->plug( menu );
	}
	~ArtsControlAppletPrivate() {
		if ( sv ) SV();
		if ( svinline ) SVinline();
		if ( am ) AM();
		if ( mmv ) MMV();
		if ( ev ) EV();
		if ( mtv ) MTV();
	}
public slots:
	void AM(); // AudioManager
	void SV(); // ScopeView
	void SVinline(); // ScopeView inline
	void ASV(); // ArtsStatusView
	void MMV(); // MidiManagerView
	void EV(); // Environment
	void MTV(); // MediaTypesView
	void moreBars();
	void lessBars();
	void styleNormalBars();
	void styleFireBars();
	void styleLineBars();
	void styleLEDs();
	void styleAnalog();
	void styleSmall();
};

class VolumeSlider : public QWidget {
   Q_OBJECT
private:
	Arts::StereoVolumeControl _volume;
	KArtsFloatWatch *_volumewatch;
	QLabel *_label;
	QSlider *_slider;
	QBoxLayout *_layout;
	bool _inUpdate;
public slots:
	void setVolume( int n ) { if ( !_inUpdate ) _volume.scaleFactor( ( n-200 ) / -100.0 ); }
	void setVolume( float n ) { _inUpdate=true; _slider->setValue( int( 200 - n * 100 ) ); _inUpdate=false; }
	void setOrientation( Qt::Orientation /*o*/ ) { /*_slider->setOrientation( o );*/ }
public:
	VolumeSlider( Arts::StereoVolumeControl volume, QWidget *parent, const char *name )
	  : QWidget( parent,name )
	  , _volume( volume )
	  , _inUpdate( false )
	{
		_volumewatch = new KArtsFloatWatch( _volume, "scaleFactor_changed", this );
		_label = new QLabel( name, this );
		_label->setFont( QFont( "Helvetica", 9 ) );
		_label->setAlignment( Qt::AlignCenter );
		_slider = new QSlider( 0, 200, 10, int( _volume.scaleFactor()*100 ), Qt::Vertical, this );
		_slider->setTickmarks( QSlider::Right );
		connect( _slider, SIGNAL( valueChanged( int ) ), this, SLOT( setVolume( int ) ) );
		connect( _volumewatch, SIGNAL( valueChanged( float ) ), this, SLOT( setVolume( float ) ) );

		_layout = new QBoxLayout( this, QBoxLayout::TopToBottom );
		_layout->addWidget( _label );
		_layout->addWidget( _slider );
	}
};

// vim: sw=4 ts=4
#endif
